# client methods called by server (rev_foo)
class GameClient

  def recv_connection_established
    @log.info "Connection to #{$serveruri} established"
    @ui.connection_made
    netsend :recv_register_player, @playername
  end

  def recv_server_disconnect(reason)
    @session = nil
    @ui.write_cmsg "Server disconnected (#{reason}). Game over, man!"
    @ui.recv_server_disconnect
  end

  def recv_cmsg(message)
    @ui.write_cmsg message
  end
 
  def recv_robot(robot)
    $game.robot = robot
  end
  
  def recv_game_state(state)
    $game.set_state(state)
  end

  # TODO fix this for multiplayer
  def recv_game_created(state)
    $game.set_state(state)
    begin
      $game.xml_board, $game.board = BoardFactory.create_from_string($game.xml_board)
      @ui.recv_game_created
    rescue REXML::ParseException => e
      @ui.write_cmsg "Can't load up this board... sorry"
      @log.error "Couldn't load up XML board from server"
      @log.debug {$game.xml_board}
    rescue Exception => e
      @ui.write_cmsg "I don't understand this board... sorry"
      @log.error "BoardFactory failed to create board from XML"
      @log.debug {e}
    end
  end

  def recv_init_game
    @log.info "init client game"
    @ui.start_game
  end

  def recv_turn_history
    @log.debug $game.turn_history
    @ui.draw_turn
  end

  def recv_game_over
    @log.info "Game Over"
    return unless $game.player.in_game?
    @ui.game_over
  end

  def recv_start_programming_phase
    @log.debug "starting programming phase with hand #{$game.robot.hand}"
    @ui.draw_hand
  end

  # uhm.. well... not much to do
  def recv_game_over
  end
end
