class Conveyor < Floor
  attr_reader :speed, :direction, :turns, :left, :right, :straight
  
  def initialize(hash={})
    @speed = ifrom_hash(hash, :speed)
    # this defaults to north, by virtue of Facing default
    @direction = (hash[:direction] or Facing.n)
    @direction = Facing.new(@direction) if not @direction.kind_of?(Facing)
    # the turn list is about how the conveyor turns
    # must turn a list of strings into standard data
    @turns = sfrom_hash(hash, :turns)
    if @turns.nil?
      @turns = ["s"] 
    else
      @turns = list_split(@turns)
    end
    @turns.map {|t| t.downcase! }
    for turn in @turns
      @straight = true if turn == "straight" or turn == "s"
      @left = true if turn == "left" or turn == "l"
      @right = true if turn == "right" or turn == "r"
      turn = "s" if @straight
      turn = "l" if @left
      turn = "r" if @right
    end
    @turns.sort!
  end

  def execute(phase, priority, row, col)
    ret = false
    ret = case priority
    when 1 then move_robots(row, col) if @speed == 2
    when 2 then 
      rotate_robots(row, col) if @speed == 2
      move_robots(row, col)
    when 5 then rotate_robots(row, col)
    end
    ret
  end

  # refactor candidate
  def move_robots(row, col)
    robots = $game.get_robots(row, col) 
    for r in robots
      if r.push(@direction)
        $game.record(Action::MOVE) # because push doesn't record
      end
    end
  end

  # refactor candidate
  def rotate_robots(row, col)
    return false unless @left or @right
    res1, res2 = false, false
    robots = $game.get_robots(row, col)
    for robot in robots
      lastsq = $game.board.square_at(*robot.prevpos)
      if @left
        leftsq = $game.board.next_square2(@direction.left, row, col)
        if lastsq == leftsq
          robot.lturn
          res1 = true
        end
      end
      if @right
        rightsq = $game.board.next_square2(@direction.right, row, col)
        if lastsq == rightsq
          robot.rturn
          res2 = true
        end
      end
    end
    return true if res1 or res2
    false
  end

  def key
    self.class.to_s + @speed.to_s + @turns.join + @direction.to_s
  end
end

class SquareFactory
  # passed elements of 'conveyor'
  def self.decode_conveyor(xml)
    args = {
      :speed => xml.value_of("speed").to_i,
      :direction => Facing.new(xml.value_of("direction")),
      :turns => xml.value_of("turns")
    }
    Conveyor.new(args)
  end
end
