# lasers are responsible for drawing lasers and for damaging robots
class Lasers < Element
  attr_accessor :n, :w, :s, :e

  def initialize
    @n, @w, @s, @e = 0, 0, 0, 0
  end

  def has_lasers?
    return (@n > 0 or @w > 0 or @s > 0 or @e > 0)
  end

  def execute(phase, priority, row, col)
    return unless priority == 6
    for r in $game.game_robots
      if r.row == row and r.col == col
        r.take_damage( @n + @w + @s + @e )
        $game.record(Action::LASERFIRE)
      end
    end
  end

  def key; "#{self.class}#{@n}#{@w}#{@s}#{@e}" end
  def is_source?; false end

end

class LaserSource < Lasers
  attr_reader :number, :direction

  def initialize(hash)
    @number = (hash[:number] or 1)
    @direction = (hash[:direction] or Facing.n)
    reinitialize
  end

  # when lasers have changed due to walls
  def reinitialize
    @n, @w, @s, @e = 0, 0, 0, 0
    case @direction
    when N then @n = @number
    when W then @w = @number
    when S then @s = @number
    when E then @e = @number
    end
  end
  
  # put a Lasers element in each forward square until a wall is encountered
  # also add a wall to the square if source isn't mounted yet
  # the case where a source is put on a square with pre-existing Lasers
  # is handled by the board editor
  def init_deps(board, square)
    if not square.wall_at?(@direction.rear)
      square.walls.add(@direction.rear)
    end
    # now add lasers to the following squares
    nexts = square
    while not board.wall_in_way?(@direction, nexts)
      nexts = board.next_square(@direction, nexts)
      break if nexts.kind_of?(NoSquare)
      add_lasers(nexts)
    end
  end

  def add_lasers(square)
    lasers = square.get(Lasers)
    lasers = Lasers.new if lasers.nil?
    lasers.n += @number if @direction == N
    lasers.w += @number if @direction == W
    lasers.s += @number if @direction == S
    lasers.e += @number if @direction == E
    square.add lasers
  end

  def is_source?; true end
  def key; "#{self.class}#{@direction}#{@n}#{@w}#{@s}#{@e}" end

end

class SquareFactory
  # passed elements of 'laser'
  def self.decode_laser(xml)
    args = {
      :number => xml.value_of("number").to_i,
      :direction => Facing.new(xml.value_of("direction"))
    }
    LaserSource.new(args)
  end
end
