class Square
  include Tools
  attr_accessor :row, :col, :floor, :walls, :elements, :x, :y

  def initialize(hash)
    @row, @col = ifrom_hash(hash, :row, :col)
    @walls = (hash[:walls] or Walls.new)
    @floor = (hash[:floor] or PlainFloor.new)
    @elements = (hash[:elements] or [])
    @x = $game.theme['square_size'] * @col
    @y = $game.theme['square_size'] * @row 
    @dirty = true
  end

  # add an element, replaces old one of the same class
  def add(element)
    e = get(element.class)
    @elements.delete(e) if e
    @elements << element
  end

  # get an element of type class. ex: LaserSource
  def get(eclass)
    @elements.find { |e| e.kind_of?(eclass) }
  end
  # shortcut
  def delete(element)
    @elements.delete(element)
  end

  def ==(other)
    @row == other.row and @col == other.col
  end

  def ===(other)
    self == other
  end

  def dirty?; @dirty end
  def dirty
    @dirty = true
  end
  def clean
    @dirty = false
  end

  def immediate_effects
    @floor.immediate_effects(@row, @col)
    for e in @elements
      e.immediate_effects(@row, @col)
    end
  end

  # run square elements at phase and priority
  def execute(phase, priority)
    @walls.execute(phase, priority, @row, @col)
    @floor.execute(phase, priority, @row, @col)
    for e in @elements
      e.execute(phase, priority, @row, @col)
    end
  end

  def end_of_turn
  end

  # convenience shortcuts
  def has_walls?
    @walls.has_walls?
  end

  def wall_at?(dir)
    @walls.wall_at?(dir)
  end
  
  def has_pit?
    @floor.kind_of?(Pit)
  end

  def has_lasers?
    return false if get(Lasers).nil?
    true
  end

  # init data that depend on other elements
  def init_deps(board)
    @walls.init_deps(board, self)
    @floor.init_deps(board, self)
    for e in @elements
      e.init_deps(board, self)
    end
  end

  def to_s
    buff = "<#{self.class}:#{@row},#{@col}"
    buff += ":Walls=#{@walls.to_s}" if has_walls?
    return buff + ">"
  end
  def inspect; to_s end

end

# handy null object
class NoSquare < Square
  def get(what); nil end
  def initialize; end
  def to_s; "<NoSquare>" end
  def dirty?; false end
  def draw(gui); end
  def has_walls?; false end
  def wall_at?(dir); false end
  def has_pit?; false end
  def has_lasers?; false end
  def immediate_effects; false end
  def execute(*args); 0 end
  def ==(other); false end
  def ===(other); false end
end
