# Most objects have a facing, this class encapsulates
# the common transformations and instance variables
class Facing
  attr_reader :name, :symbol
  @@facings = %w{ n w s e }

  def initialize(name)
    if name.nil?
      raise ArgumentError, "Nil Facing not allowed", caller[1..-1]
    end
    name.strip!
    if not name =~ /^[nwse]$/
      raise ArgumentError, "Bad Facing: #{name}", caller[1..-1]
    end
    @name = name
  end

  # comparison methods
  def ==(obj)
    @name == obj.to_s
  end
  def ===(obj); self == obj end
  def hash; @name.hash end

  # transform methods
  def right!
    case @name 
    when "n" then @name = "e"
    when "w" then @name = "n"
    when "s" then @name = "w"
    when "e" then @name = "s"
    end
    self
  end
  def left!
    3.times { right! }
    self
  end
  def rear!
    2.times { right! }
    self
  end

  # copy transform methods
  def right; clone.right! end
  def left; clone.left! end
  def rear; clone.rear! end

  # given a bunch of facings, clone and rotate all until the reference
  # is facing north
  def orient_north(*facings)
    return facings if @name == 'n'
    faces = [self.clone] + facings.collect {|f| f.clone }
    while faces[0] != 'n' do faces.each {|f| f.right!} end
    faces.shift
    faces
  end

  def inspect; @name end
  def to_s; @name end

  class << self
    def each_name
      @@facings.each {|name| yield name }
    end
    def each
      each_name {|name| yield Facing.new(name)}
    end
    def all
      @@facings.collect{|name| Facing.new name}
    end
    def random
      Facing.new @@facings[rand(4)]
    end
    def n; Facing.new 'n' end
    def w; Facing.new 'w' end
    def s; Facing.new 's' end
    def e; Facing.new 'e' end
  end
end

N, W, S, E = Facing.all
