class Deck
  attr_reader :cards, :discards
  def initialize
    @cards = []
    @discards = []
    # default deck
    cardnames = %w{uturn rturn lturn back_1 fwd_1 fwd_2 fwd_3}
    numcards = [6, 18, 18, 6, 18, 12, 6]
    cardranks = [1, 2, 2, 3, 4, 5, 6]
    allcards = []
    cardnames.size.times {|i| allcards << Card.new(cardnames[i], cardranks[i])} 
    allcards.size.times {|i| @cards += allcards[i] * numcards[i]}
    incr, step = 1, 10
    for card in @cards
      card.priority = incr * step
      incr += 1
    end
  end
  def size; @cards.size end
  def to_s; "Deck: " + @cards.join(", ") end

  def shuffle
    srand
    @cards += @discards; @discards.clear
    # iterate and swap each element with a random one after it
    for i in 0...(@cards.size-1)
      j = rand(@cards.size - i) + i
      elmt = @cards[i]
      @cards[i] = @cards[j]
      @cards[j] = elmt
    end
  end

  # here we change the registers according to damage
  # and deal a hand
  def deal_hand(robot)
    return if not robot.alive?
    robot.registers.change(num_locked(robot))
    robot.registers.clear_unlocked_cards
    @discards += robot.hand
    robot.old_hand = robot.hand
    robot.hand.clear
    num_deal(robot).times {robot.hand.push @cards.pop}
  end
  # always deal up to 9 - possible to customize
  def num_deal(robot)
    num = robot.maxdamage - robot.damage 
    return 9 if num > 9
    num
  end
  # there are always 5 registers - possible to generalize
  def num_locked(robot)
    return 0 if robot.maxdamage - robot.damage >= 5
    robot.damage - robot.maxdamage + 5
  end
end

