# game server methods for game, excluding turn-specific
class GameServer

  # entry point when players are ready_to_play
  def start_game
    @log.info "Game is #{$game.gametype.class}"
    @log.debug {$game.gametype}
    $game.game_running = true
    $game.deck.shuffle
    #assign_robots
    $game.gametype.adjust($game)
    $game.game_robots.each {|r| r.virtual = true}
    init_client_game
    do_turn
  end

  def init_client_game
    state = DeltaState.new :game_robots, :turn
    netsend_all :recv_game_state, state
    for robot in $game.game_robots
      netsend robot.clientid, :recv_init_game
    end
  end

  # not necessary.. what else to do? maybe images? client side
  def assign_robots
    return
    assign = $robots.clone
    for player in $game.game_players
      player.robot = Robot.new(player.prefname, assign.shift)
    end
    $game.robots.size.times {|i|
      $game.robots[i].robot_name = $robots[i]
    }
  end

  # entry point at begining of each turn
  def do_turn
    $game.game_robots.each {|p| p.program_complete = false}
    do_programming_phase
  end
  
  def do_programming_phase
    # pointless.. limit the number of players
    #if $game.game_robots.size * 10 > $game.deck.size
    $game.deck.shuffle
    deal_hands
  end

  def deal_hands
    $game.game_robots.each do |robot|
      $game.deck.deal_hand(robot)
      netsend robot.clientid, :recv_robot, robot
      netsend robot.clientid, :recv_start_programming_phase
    end
  end

  # client calls this when submitting a hand
  def recv_play_hand(clientid, hand)
    return if $game.game_running == false
    robot = $game.game_robots.find {|r| r.clientid == clientid}
    return if robot.nil?
    if robot.program_complete
      send_cmsg clientid, "You did that already."
      return
    end
    robot.program_complete = true
    begin
      robot.handle_play(hand)
    rescue CheaterError => ce
      @log.warn "Cheater detected: #{ce.message}"
      kick_robot(clientid, "for cheating")
      return
    end
    # wait until all robots have finished programming
    for r in $game.game_robots
      return if not r.program_complete
    end
    trap_any("Server incapable of handing game logic!") { compute_turn }
  end

end
