# turn stuff here
class GameServer
  def compute_turn
    $game.turn += 1
    $game.turn_history = []
    @log.debug "computing turn #{$game.turn}"
    $game.record(Action::PAUSE)
    (1..5).each {|num| compute_phase(num) }
    end_of_turn
    prep_next_turn
    $game.game_history << $game.turn_history.clone
    send_turn_history
    end_game_by_objective if $game.gametype.game_over?
  end

  # TODO: animate all robots simultaneously after each phase
  def compute_phase(phase)
    @log.debug "computing phase #{phase}"
    @log.debug "  - robot movement"
    $game.phase = phase
    robots_move(phase)
    board_components_act(phase)
    resolve_laser_fire
    end_of_phase
  end

  def robots_move(phase)
    $game.live_robots.sort.reverse.each {|r| r.execute_phase(phase)}
  end

  def board_components_act(phase)
    for priority in 1..6
      action_squares = $game.board.squares_with_robots
      for s in action_squares
        s.immediate_effects # need to do this in case robot is pushed on pit 
        s.execute(phase, priority)
      end
    end
    $game.board.squares_with_robots.each {|s| s.immediate_effects}
  end

  # robots only TODO: debug case where fired > 0 for one robot
  def resolve_laser_fire
    fired = $game.live_robots.collect{|r| r.fire_laser}
    fired.delete(false)
    $game.record(Action::LASERFIRE) if fired.size > 0
    $game.live_robots.each {|r| r.die_if_dead!}
  end

  # kind of redundant
  def end_of_phase
    return
    $game.board.squares_with_robots.each {|t| t.end_of_phase}
  end

  def end_of_turn 
    @log.debug "computing end of turn"
    $game.board.squares_with_robots.each {|t| t.end_of_turn}
    toggle_virtual
  end

  def toggle_virtual
    $game.virtual_robots.each {|r|
      others = $game.get_robots(r.row, r.col)
      r.virtual = true if others.size > 1
    }
  end

  def prep_next_turn
    $game.dead_robots.each {|r| r.revive}
    toggle_virtual
    $game.record(Action::PAUSE)
    # remove any robots with no lives
    remove = []
    for robot in $game.game_robots
      remove << robot if robot.lives == 0
    end
    remove.each { |r| $game.game_robots.delete(r) }
  end

  def end_game_by_objective
    @log.info "Game Over"
    shout $game.gametype.text($game)
    $game.end_game
    netsend_all :recv_game_over
  end

  def send_turn_history
    public_state = DeltaState.new :turn_history, :turn
    netsend_all :recv_game_state, public_state
    netsend_all :recv_turn_history
    # now start the turn over again (which will end up dealing cards)
    do_turn if not $game.gametype.game_over?
  end

end
