class Robot
  # send receivers (card name invokes a method)
  # the movement must be done in steps for animation purposes
  def fwd_1; any_move_fwd end
  def fwd_2; 2.times {any_move_fwd} end
  def fwd_3; 3.times {any_move_fwd} end
  def back_1; any_backup end
  def uturn; @facing.rear!; any_turn end
  def rturn; @facing.right!; any_turn end
  def lturn; @facing.left!; any_turn end

  def any_move_fwd # might want to record a failed move for sound effect
    end_of_move if move_forward
  end

  def any_backup
    end_of_move if backup
  end

  # things to do after movement
  # * deal with any immediate tile effects, such as pit death and out
  #   of bound death and elements that are triggered by stepping on them
  #   die!(Action::FALLOFF) if $game.board.out_of_bounds?(row, col)
  def end_of_move 
    $game.record(Action::MOVE) # show all robot movements first
    $game.board.squares_with_robots.each {|t| t.immediate_effects}
  end

  def any_turn
    $game.record(Action::ROTATE)
  end
    
  # this is a recursive push. i.e., this will push any other robots in the
  # way of the one being pushed. returns false if the push failed
  def push(direction)
    return case direction
    when @facing then move_forward
    when @facing.rear then backup
    when @facing.left then move_left
    when @facing.right then move_right
    end
  end

  # The canonical move function. All other moves are a combination of
  # rotations and this. Returns true if move successful.
  # accepts an "unwinding" block to run after movement
  def move_forward
    return false if @dead
    # this square
    square = $game.board.square_at(@row, @col)

    # first things first, fail if wall in way
    if $game.board.wall_in_way?(@facing, square)
      yield if block_given?
      return false
    end

    # next row and col
    nrow, ncol = $game.board.next_square_pos(@facing, @row, @col)

    # fall in pit our out of bounds
    pit_there = $game.board.square_at(nrow, ncol).get(Pit)
    if pit_there or $game.board.out_of_bounds?(nrow, ncol)
      execute_move(nrow, ncol)
      yield if block_given?
      return true
    end
    
    # push any robots
    robot = $game.get_robot(nrow, ncol)
    if robot # if there's a robot ahead
      if robot.push(@facing) # attempt to push robot
        # push succeeded. now, actually execute the move
        execute_move(nrow, ncol)
      else
        yield if block_given?
        return false
      end
    else execute_move(nrow, ncol)
    end
    yield if block_given?
    return true
  end

  def execute_move(nrow, ncol)
    @prevpos = [@row, @col]  # record last
    @row, @col = nrow, ncol    # update position
    die!(Action::FALLOFF) if $game.board.out_of_bounds?(@row, @col)
  end
    
  def backup
    @facing.rear!
    move_forward {@facing.rear!}
  end

  def move_left
    @facing.left!
    move_forward {@facing.right!}
  end

  def move_right
    @facing.right!
    move_forward {@facing.left!}
  end
end
