class Register
  include Comparable
  attr_accessor :card
  attr_reader :num
  def initialize(num)
    @num = num
    @locked = false
  end
  def <=>(other)
    @num <=> other.num
  end
  def succ
    Register.new(num.succ)
  end
  def locked?; @locked end
  def unlocked?; not @locked end
  def lock; @locked = true end
  def unlock; @locked = false end
  def to_s; "<Register#{@num}:locked=#{@locked}:#{@card}>" end
  def inspect; to_s end
end

class Registers
  attr_reader :registers
  def initialize
    @registers = (Register.new(1)..Register.new(5)).to_a
  end
  # if we're full of cards
  def full?
    @registers.each {|r| return false if r.card.nil?}
    true
  end
  # lock or unlock registers according to the change in locked registers
  def change(new_locked_size)
    diff = new_locked_size - locked.size
    diff.times{lock_top} if diff > 0
    diff.abs.times{unlock_top} if diff < 0
  end
  def lock_top
    return if @registers[0].locked?
    @registers.reverse.find{|r| r.unlocked?}.lock
  end
  def unlock_top
    return if @registers[4].unlocked?
    @registers.find{|r| r.locked?}.unlock
  end
  def [](key)
    return @registers[key-1] if key.kind_of?(Integer)
    if key.type == Range and key.first.kind_of?(Integer)
      return @registers[key.first-1..key.last-1]
    end
    nil
  end
  def locked
    @registers.collect{|r| (r.locked? ? r : nil)}.compact
  end
  def unlocked
    @registers.collect{|r| (r.unlocked? ? r : nil)}.compact
  end
  def each
    @registers.each {|r| yield r}
  end
  def each_unlocked
    unlocked.each {|r| yield r}
  end
  def each_locked
    locked.each {|r| yield r}
  end
  def each_card
    @registers.each {|r| yield r.card}
  end
  def cards
    @registers.collect {|r| r.card}
  end
  def unlocked_cards
    @registers.collect{|r| (r.unlocked? ? r.card : nil)}.compact
  end
  def locked_cards
    @registers.collect{|r| (r.locked? ? r.card : nil)}.compact 
  end
  def clear_unlocked_cards
    each_unlocked {|r| r.card = nil} 
  end
  # cards is guaranteed to be size of unlocked
  def assign(cards)
    unlocked.size.times {|i| @registers[i].card = cards[i]}
  end
  def to_s; @registers.inspect end
  def inspect; to_s end
end
