# server methods called by client
# specifically those having to do with joining players
class GameServer

  def recv_register_player(clientid, prefname=nil)
    prefname = "Player#{clientid}" if prefname.nil?
    @log.debug "Creating player #{prefname} for #{clientid}"
    player = Robot.new(clientid, prefname)
    $game.robots[clientid] = player
    netsend(clientid, :recv_robot, player)
    @log.debug "Created and sent player for #{clientid}"
    announce_join(clientid, prefname)
  end

  def announce_join(clientid, prefname)
    extra = "arrives"
    if $game.game_running
      init_observer(clientid)
      extra = " the kibitzer arrives"
    end
    shout "#{prefname} #{extra}"
    welcome_player(clientid)
  end

  def welcome_player(clientid)
    message = "Welcome to the development botfrenzy gameserver!\n\n"
    if $game.game_running?
      message += "A game is already in progress, so please sit back "
      message += "and watch the action. "
    elsif $game.game_created?
      message += "A game is about to begin. Select a robot if you want to join."
      message += $game.gametype.to_s
    else
      message += "There is no game defined. Please create a game or wait for "
      message += "someone to create one."
    end
    send_cmsg(clientid, message)
  end

  def recv_client_disconnect(clientid)
    name = $game.robots[clientid].name
    shout "#{name} left the game."
    remove_all_traces(clientid)
  end

  def kick_player(clientid, reason="just because")
    shout "#{$game.robots[clientid].name} was kicked " + reason + "."
    remove_all_traces(clientid)
  end

  # players who join a game already in progress become observers
  # TODO: this is complex.. what of robots who don't join?
  def init_observer(clientid)
    @log.info "init observer"
    public_state = DeltaState.new :game_robots, :turn, :turn_histories
    netsend clientid, :recv_game_state, public_state
    netsend clientid, :recv_init_game
    netsend clientid, :recv_animate_phase
  end

  def trap_any(reason)
    begin yield
    rescue Exception => e
      @log.fatal {e}
      shutdown reason
    end
  end

  # needs work
  def recv_change_robot(clientid, name)
    return if game_already(clientid)
    if $robots.include? name
      $game.players[clientid].robot_name = name
    else
      send_cmsg clientid, "Sorry, #{name} is not a valid robot."
    end
  end 

  def recv_change_name(clientid, nick)
    shout "#{$game.players[clientid].nick} changed name to #{nick}"
    synch { $game.players[clientid].nick = nick }
  end

end
