require 'bf/state/themestate'
require 'bf/state/snapshot'

class GameState < ThemeState
  attr_accessor :phase, :turn, :board, :xml_board, :turn_history,
                :game_history, :robots, :gametype, :deck, :robot
  attr_accessor :game_robots, :game_running, :game_created

  # robots are keyed to the sessionid
  def initialize
    super
    @robots = Hash.new
    end_game
  end
  def end_game
    @game_running = false
    @game_created = false
    @game_robots = []
    @turn = 0
    @deck = Deck.new
    @turn_history = []
    @game_history = []
    @board = nil
    @xml_board = nil
    @gametype = nil
  end
  def game_running?; @game_running end 
    def game_created?; @game_created end
    def all_robots; @robots.values end
    def live_robots
      @game_robots.collect{|r| r if r.alive?}.compact
    end
    def dead_robots
    @game_robots.collect{|r| r if not r.alive?}.compact
  end
  def virtual_robots
    @game_robots.collect{|r| r if r.virtual?}.compact
  end
  def solid_robots
    @game_robots.collect{|r| r if not r.virtual?}.compact
  end
  # returns the non-virtual robot at row, col or nil
  def get_robot(row, col)
    solid_robots.each {|r| return r if [r.row, r.col] == [row, col]}
    return nil
  end
  # gets the set of all robots at a position, including virtual
  def get_robots(row, col)
    @game_robots.collect {|r| r if [r.row, r.col] == [row, col]}.compact
  end
  def record(action=Action::UNDEFINED)
    @turn_history << Snapshot.new(action)
  end
  def remove(clientid)
    r = @robots.delete(clientid)
    @game_robots.delete(r)
    record(action=Action::PLAYERQUIT)
  end
  def record(action=Action::UNDEFINED, *subjects)
    @turn_history << Snapshot.new(action, *subjects)
  end
end

# whenever the game needs to send the client some data, it will wrap
# it up in this class, which gets marshalled. Not all data needs to
# be wrapped up, just the difference in state
# usage: ClientDeltaState.new(:board, :robots)
class DeltaState
  attr_accessor :phase, :turn, :board, :xml_board, :turn_history,
                :game_history, :game_robots, :gametype, :robot

  # populate the requested fields with data from the actual game state
  def initialize(*fields)
    fields.each { |f| send("#{f.to_s}=", $game.send(f)) }
  end
end
