class ThemeState
  attr_accessor :images, :display, :theme, :theme_name, :theme_path
  
  # initialize the theme and image hash by default
  def initialize
    @theme = Hash.new
    @theme_name = "default"
    load_theme(@theme_name)
    @images = Hash.new
  end

  # returns the path to the theme key
  def theme_file(key)
    File.join(@theme_path, @theme[key])
  end

  # this is a client particular
  def load_theme(name)
    @theme_name = name
    # need to redraw board, etc. for now, just set the path
    @theme_path = File.join( Defs::THEME_PATH, @theme_name )
    file = File.join(@theme_path, "theme")
    load_theme_file(file)
  end

  # also a client particular
  def load_theme_file(file)
    hash = Hash.new
    File.foreach(file) {|line|
      md = /(\w*)\s*=\s*(\w*(\.\w*)?)/.match(line)
      next if md.nil? # raise exception maybe?
      key, value = md[1], md[2]
      hash[key] = value
    }
    # post-validate: change data to numbers and check all vars?
    hash['square_size'] = hash['square_size'].to_i
    @theme.replace hash
  end

  def load_image(key)
    file = File.join( @theme_path, @theme[key] )
    SDL::Surface.load( file ) 
  end

  def load_font(key, size)
    file = File.join( @theme_path, @theme[key] )
    font = SDL::TTF.open(file, size)
    font.style = SDL::TTF::STYLE_NORMAL
    font
  end


end
