# this file is usually run independently of the game
# later on, it will be refactored to incorporate the whole
# game. there should be only one entry point for the game
# and it should be run from the bin/ directory

require 'sdl'

require 'bf/definitions'
require 'bf/facing'
require 'bf/elements/include'
require 'bf/elements/sdl/include'
require 'bf/factory'
require 'bf/elements/xml'
require 'bf/state/clientstate'

require 'gui/app'
require "bfeditor/boarddisplay"
require "bfeditor/boardactions"
require "bf/games/include"

# load up the game resurces
$game = ClientGameState.new

# This class wraps up all the major widgets
class BFEditor < MainWindow
  MAXROWS, MAXCOLS = 12, 12
  attr_reader :square_size, :tool_w, :savedialogue

  def initialize(hash)
    # save file stuff
    @save_file = sfrom_hash(hash, :save_file)
    @save_file = File.join(".","boards","game_board.xml") if @save_file.nil? or @save_file.strip.empty?

    # size of the whole window
    @square_size = $game.theme['square_size']
    pad = Theme['pad'] * 2
    @rows, @cols = ifrom_hash(hash, :rows, :cols)
    @tool_w = @square_size * 4 + pad * 4
    hash[:w] = @square_size * MAXCOLS + pad * 4 + @tool_w
    hash[:h] = @square_size * MAXROWS + pad * 4 + 33
    hash[:label] = "BotFrenzy Editor"
    super(hash)

    # size of the board display
    bw = @cols * @square_size
    bh = @rows * @square_size
    bx = pad + (@square_size * MAXCOLS - bw) / 2
    by = pad + (@square_size * MAXROWS - bh) / 2 + 33
    @boarddisplay = BoardDisplay.new(:rows=>@rows, :cols=>@cols, 
      :x=>bx, :y=>by, :w=>bw, :h=>bh,
      :file=>hash[:file]
    )
    @radio = HToolBar.new(:x=>@w-@tool_w, :y=>33, :w=>@tool_w)
    add @boarddisplay, @radio
    @boarddisplay.radio = @radio

    init_menu
  end

  def init_sdl
    super()
    draw_init
    load_buttons
  end

  def draw_init
    pad = Theme['pad']
    @surface.fill_rect(0,0,@w,@h, 0)
    draw_subborder(@surface, Theme['window_border'], pad*2,
      0, 33, @square_size * MAXROWS + pad*4, @square_size * MAXCOLS + pad * 4)
  end

  def load_buttons
    path = File.join(Defs::THEME_PATH,$game.theme_name)
    icons = []
    icons << new_button(path, "img_plain")
    icons << new_button(path, "img_pit")

    icons << new_button(path, "img_wall")

    # simply creating the objects generates the images in $game.images[img_key]
    w = Walls.new(Facing.n, Facing.w)
    w.init_sdl(@surface)
    wi = StickyIconButton.new(:label=>"img_wall_nw")
    wi.icon = $game.images[w.image_key]
    icons << wi

    w = Walls.new(Facing.w, Facing.e)
    w.init_sdl(@surface)
    wi = StickyIconButton.new(:label=>"img_wall_we")
    wi.icon = $game.images[w.image_key]
    icons << wi

    icons << FillSquare.new(:w=>@square_size, :h=>@square_size)

    icons << StickyIconButton.new(:imagename=>File.join(path, "rotate_wall_icon.png"), :label=>"rotate_wall")
    icons << StickyIconButton.new(:imagename=>File.join(path, "rotate_conveyor_icon.png"), :label=>"rotate_conveyor")

    # conveyors, which have a simple naming scheme
    cnames = %w{l r lr s ls rs}
    for i in 1..2
      for cname in cnames
        sym = "img_conveyor#{i}_#{cname}"
        icons << new_button(path, sym)
      end
    end

    for i in 1..3
      sym = "img_laser#{i}_source"
      icons << new_button(path, sym)
    end

    @radio.add *icons
  end

  def new_button(path, key)
    StickyIconButton.new(:imagename=>File.join(path,$game.theme[key]),
      :label=>key)
  end

  def init_menu
    @menu = Hbox.new
    sb = LabelButton.new(:label=>"Save")
    qb = LabelButton.new(:label=>"Quit")
    @menu.add sb, qb
    add @menu

    def sb.on_selected(event)
      @parent.parent.save
    end
    def qb.on_selected(event)
      @parent.parent.quit
    end

    # for saving
    @savedialogue = FixedWindow.new(:x=>(@w-400)/2, :y=>(@h-100)/2, :w=>400, :h=>100, :title=>"Save Board")
    h = Hbox.new(:w=>350, :h=>40, :x=>10, :y=>10)
    l = Label.new(:label=>"File: ", :x=>10, :y=>10)
    @t = TextInput.new(:textwidth=>20,:x=>10,:y=>10)
    def @t.on_enter
      @parent.parent.parent.save_as(@buff)
      super
    end
    h.add l, @t
    @savedialogue.add h
    @savedialogue.hide
    add @savedialogue
  end
  
  def save
    @savedialogue.show
    @savedialogue.focus
    @t.select
  end
  def save_as(fname)
    if not fname.strip.empty?
      begin
        f = File.open(fname.strip, "w")
        f.print @boarddisplay.board.to_xml
      rescue Exception => e
        f.close if f
        # TODO: show error
      end
    end
    @savedialogue.unfocus
    @savedialogue.hide
    # hmm
    draw_init
    @boarddisplay.dirty
    @menu.dirty
    @radio.dirty
  end
end
