# actions go here
class BoardDisplay
  # every board action is a method stored in a hash table keyed by
  # the event label (the tool widget name)
  def init_actions
    @actions = {
      "img_plain" => method(:action_add_plain),
      "img_pit" => method(:action_add_pit),
      "img_wall" => method(:action_add_wall),
      "img_wall_nw" => method(:action_add_wall_nw),
      "img_wall_we" => method(:action_add_wall_we),
      "rotate_wall" => method(:action_rotate_wall),
      "img_conveyor1_l" => method(:action_add_conveyor),
      "img_conveyor1_r" => method(:action_add_conveyor),
      "img_conveyor1_lr" => method(:action_add_conveyor),
      "img_conveyor1_s" => method(:action_add_conveyor),
      "img_conveyor1_ls" => method(:action_add_conveyor),
      "img_conveyor1_rs" => method(:action_add_conveyor),
      "img_conveyor2_l" => method(:action_add_conveyor),
      "img_conveyor2_r" => method(:action_add_conveyor),
      "img_conveyor2_s" => method(:action_add_conveyor),
      "img_conveyor2_rs" => method(:action_add_conveyor),
      "img_conveyor2_ls" => method(:action_add_conveyor),
      "img_conveyor2_lr" => method(:action_add_conveyor),
      "rotate_conveyor" => method(:action_rotate_conveyor),
      "img_laser1_source" => method(:action_add_laser),
      "img_laser2_source" => method(:action_add_laser),
      "img_laser3_source" => method(:action_add_laser)
      #"" => method(:)
    }
  end

  def action_save(s, k)
    @parent.save
  end

  def action_add_plain(square, key)
    square.floor = PlainFloor.new
    square.walls = Walls.new
    square.elements.clear
    recalc_pit(square)
    square.init_sdl(@surface)
    recalc_lasers(square)
  end

  def action_add_pit(square, key)
    return if square.has_pit?
    square.floor = Pit.new
    square.floor.init_deps(@board, square)
    square.init_sdl(@surface)
    recalc_pit(square)
  end

  # recalculate dependancies and remake pit images
  def recalc_pit(square)
    adj, diag = @board.surrounding_squares(square)
    adj.each do |s|
      if s.has_pit?
        s.floor.init_deps(@board, s)
        s.init_sdl(@surface)
      end
    end
    diag.each do |s|
      if s.has_pit?
        s.floor.init_deps(@board, s)
        s.init_sdl(@surface)
      end
    end
  end

  def add_walls(square, *facings)
    facings.each { |f| square.walls.add(f) }
    square.init_sdl(@surface)
    recalc_lasers(square)
  end

  def rotate_walls(square, *facings)
    square.walls = Walls.new(*facings)
    square.init_sdl(@surface)
    recalc_lasers(square, true)
  end

  def action_add_wall(s,k); add_walls(s, Facing.s) end
  def action_add_wall_nw(s,k); add_walls(s, Facing.n, Facing.w) end
  def action_add_wall_we(s,k); add_walls(s, Facing.w, Facing.e) end

  def action_rotate_wall(square, key)
    return if not square.has_walls?

    type = square.walls.to_s
    if type.size == 4 # four walls, so just rotate the laser
      recalc_lasers(square, true)
      return
    end

    case type
    when "n" then rotate_walls(square, Facing.e)
    when "w" then rotate_walls(square, Facing.n)
    when "s" then rotate_walls(square, Facing.w)
    when "e" then rotate_walls(square, Facing.s)
      # L shaped
    when "nw" then rotate_walls(square, Facing.n, Facing.e)
    when "ne" then rotate_walls(square, Facing.s, Facing.e)
    when "se" then rotate_walls(square, Facing.w, Facing.s)
    when "ws" then rotate_walls(square, Facing.n, Facing.w)
      # || shaped
    when "we" then rotate_walls(square, Facing.n, Facing.s)
    when "ns" then rotate_walls(square, Facing.w, Facing.e)
      # U shaped
    when "nwe" then rotate_walls(square, Facing.n, Facing.e, Facing.s)
    when "nse" then rotate_walls(square, Facing.w, Facing.s, Facing.e)
    when "wse" then rotate_walls(square, Facing.n, Facing.w, Facing.s)
    when "nws" then rotate_walls(square, Facing.n, Facing.w, Facing.e)
    end
  end
 
  def action_add_conveyor(square, key)
    case key
    when "img_conveyor1_l"
      square.floor = Conveyor.new(:speed=>1,:turns=>"L")
    when "img_conveyor1_r"
      square.floor = Conveyor.new(:speed=>1,:turns=>"R")
    when "img_conveyor1_lr"
      square.floor = Conveyor.new(:speed=>1,:turns=>"L, R")
    when "img_conveyor1_s"
      square.floor = Conveyor.new(:speed=>1,:turns=>"S")
    when "img_conveyor1_ls"
      square.floor = Conveyor.new(:speed=>1,:turns=>"L, S")
    when "img_conveyor1_rs"
      square.floor = Conveyor.new(:speed=>1,:turns=>"R, S")
    when "img_conveyor2_l"
      square.floor = Conveyor.new(:speed=>2,:turns=>"L")
    when "img_conveyor2_r"
      square.floor = Conveyor.new(:speed=>2,:turns=>"R")
    when "img_conveyor2_s"
      square.floor = Conveyor.new(:speed=>2,:turns=>"S")
    when "img_conveyor2_lr"
      square.floor = Conveyor.new(:speed=>2,:turns=>"L, R")
    when "img_conveyor2_ls"
      square.floor = Conveyor.new(:speed=>2,:turns=>"L, S")
    when "img_conveyor2_rs"
      square.floor = Conveyor.new(:speed=>2,:turns=>"R, S")
    end
    square.init_sdl(@surface)
  end

  def action_rotate_conveyor(square, key)
    if square.floor.kind_of?(Conveyor)
      square.floor.direction.right!
      square.init_sdl(@surface)
    end
  end

  def action_add_laser(square, key)
    case key
    when "img_laser1_source" then add_laser_source(square, 1, Facing.n)
    when "img_laser2_source" then add_laser_source(square, 2, Facing.n)
    when "img_laser3_source" then add_laser_source(square, 3, Facing.n)
    end
  end

  def add_laser_source(s, n, dir)
    add = true

    # destroy any present lasers or sources
    l = s.get(Lasers)
    s.elements.delete(l) if l

    # check next square for laser conflict
    ns = @board.next_square(dir, s)
    if not @board.wall_in_way?(dir, s) and ns.has_lasers?
      l = ns.get(Lasers)
      if (dir == N and l.s > 0) \
        or (dir == W and l.e > 0) \
        or (dir == S and l.n > 0) \
        or (dir == E and l.w > 0)
        # forbid adding the source
        add = false
      end
    end

    s.add(LaserSource.new(:number=>n, :direction=>dir)) if add

    # now go through each square and re-build the deps and sdl
    @board.rebuild_lasers
    @board.squares.each { |s| s.init_sdl(@surface) }
  end

  # called after walls are messed with
  def recalc_lasers(square, rotate=false)
    # if lasers are inbound on this square, rebuild lasers
    if square.has_lasers?
      l = square.get(Lasers)
      if rotate and l.is_source? # rotate the source
        add_laser_source(square, l.number, l.direction.right)
      else
        @board.rebuild_lasers
        @board.squares.each { |s| s.init_sdl(@surface) }
      end
    else # check surrounding squares too (inefficient)
      adj = @board.adjacent_squares(square)
      for a in adj
        if a.has_lasers?
          @board.rebuild_lasers
          @board.squares.each { |s| s.init_sdl(@surface) }
          return
        end
      end
    end
  end
 
end
