# this file is usually run independently of the game
# later on, it will be refactored to incorporate the whole
# game. there should be only one entry point for the game
# and it should be run from the bin/ directory

# TODO: decouple from BFEditor
class BoardDisplay < Widget
  attr_accessor :radio, :board

  def initialize(hash={})
    super(hash)

    # load or init a board
    file = hash[:file]
    if file
      xml, @board = BoardFactory.create_from_file(file)
      @squares = @board.squares
    else create_blank_board(hash)
    end

    # bindings
    connect_method(SDL::Event2::MouseButtonDown, :on_mouse_down)

    # actions
    init_actions
  end

  def create_blank_board(hash)
    rows, cols = ifrom_hash(hash, :rows, :cols)
    @squares = []
    for row in 0...rows
      for col in 0...cols
        @squares << Square.new(:row=>row, :col=>col)
      end
    end
    @board = Board.new("", "", rows, cols, @squares)
  end

  # in addition to the normal surface, init_sdl on each square
  def init_sdl
    # center smaller boards
    @surface = SDL::Surface.new(Defs::Gfx::SDLFLAGS, @w, @h, @@root_surface)
    @squares.each {|s| s.init_sdl(@surface) }
  end
  def draw(surf, px, py)
    return unless dirty?
    @squares.each {|s| s.draw(@surface)}
    surf.put(@surface, px+@x, py+@y)
    clean
  end

  # change the selected square into the tool element
  def on_mouse_down(event)
    return unless event.button == 1
    element = @radio.selected
    return if element.nil?
    square = @board.square_at_coord(event.x-@x, event.y-@y)

    if ( a = @actions[element.label] )
      a.call(square, element.label) 
      dirty
    end
  end
end
