require 'gui/include'

class MainWindow < Widget
  def initialize(hash)
    super(hash)
    @parent = RootWidget.new
  end
  def init_sdl
    SDL.init( SDL::INIT_VIDEO )
    @surface = SDL::setVideoMode(@w, @h, 32, Defs::Gfx::SDLFLAGS)
    SDL::WM.setCaption(@label, "")
    @@root_surface = @surface
    SDL::TTF.init
  end
  def focused
    @@focused_widget
  end
  def key_focused
    @@key_focused_widget
  end
  def draw
    for child in @children
      child.draw(@surface, 0, 0)
    end
    clean
    @surface.flip
  end
  def quit
    exit
  end
end

class App
  def initialize(main)
    @main = main
  end
  def run
    @main.init_all
    event_loop
  end
  # this should be optimized in C
  # along with some widget methods
  def event_loop
    while true
      if @main.dirty?
        @main.draw
      end
      event = SDL::Event2.poll
      case event
      when SDL::Event2::Quit
        @main.quit
      when SDL::Event2::MouseButtonDown, SDL::Event2::MouseButtonUp
        if not @main.focused.nil?
          bound = @main.focused.bindings[event.class]
          bound.call(event) if not bound.nil?
        else
          widget = @main.find_widget(event.x, event.y)
          next if widget.nil?
          bound = widget.bindings[event.class]
          bound.call(event) if not bound.nil?
        end
      when SDL::Event2::KeyDown #, SDL::Event2::KeyUp
        next unless @main.key_focused
        bound = @main.key_focused.bindings[event.class]
        bound.call(event) if not bound.nil?
      end
    end
  end
  def call_binding(event, widget)
    bound = widget.bindings[event.class]
    bound.call(event) if not bound.nil?
  end
end
