class Button < Widget
  PAD = 2
  attr_accessor :hicolor
  def initialize(hash={})
    super(hash)
    @hicolor = (hash[:hicolor] or 0x0000FF00)
    connect_method(SDL::Event2::MouseButtonDown, :on_mouse_down)
    connect_method(SDL::Event2::MouseButtonUp, :on_mouse_up)
    @pressed = false
    @active = false
  end
  def pressed?; @pressed end
  def init_sdl
    @hi = create_surface(@w, @h)
    @unhi = create_surface(@w, @h)
    draw_border(@hi, @hicolor, PAD)
    @surface = @unhi
  end
  def highlight
    @surface = @hi
  end
  def unhighlight
    @surface = @unhi
  end
  # called by user
  def on_selected(event)
  end
  def on_mouse_down(event)
    return if not event.button == 1
    @pressed = true
    focus
    highlight
    dirty
  end
  def on_mouse_up(event)
    return if not event.button == 1
    unfocus
    @pressed = false
    unhighlight
    dirty
    # only select on mouse up and inside
    inside = root_inside?(event.x, event.y)
    on_selected(event) if inside
  end
end

class StickyButton < Button
  def active?; @active end
  def active=(bool)
    @active = bool
    if @active
      highlight
    else
      unhighlight
    end
    dirty
  end
  def on_mouse_down(event)
    return if not event.button == 1
    @pressed = true
    focus
    if @active
      unhighlight
    else
      highlight
    end
    dirty
  end
  def on_mouse_up(event)
    return if not event.button == 1
    unfocus
    # two cases: mouse on button or mouse outside
    inside = root_inside?(event.x, event.y)
    if inside and @pressed
      @active = !@active
    end
    if @active
      highlight
      on_selected(event)
    else
      unhighlight
    end
    @pressed = false
    dirty
  end
end

class IconButton < Button
  attr_reader :imagename
  attr_accessor :icon
  def initialize(hash={})
    super(hash)
    @imagename = sfrom_hash(hash, :imagename)
  end
  def init_sdl
    @icon = SDL::Surface.load(@imagename) if not @imagename.nil?
    if @icon.nil?
      raise ArgumentError, "IconButton icon not defined!", caller[1..-1]
    end
    @w = @icon.w + PAD*2
    @h = @icon.h + PAD*2
    super
    @hi.put(@icon, PAD, PAD)
    @unhi.put(@icon, PAD, PAD)
    @parent.resized = true
  end
end

# same as above
class StickyIconButton < StickyButton
  attr_reader :imagename
  attr_accessor :icon
  def initialize(hash={})
    super(hash)
    @imagename = sfrom_hash(hash, :imagename)
  end
  def init_sdl
    @icon = SDL::Surface.load(@imagename) if not @imagename.nil?
    if @icon.nil?
      raise ArgumentError, "IconButton icon not defined!", caller[1..-1]
    end
    @w = @icon.w + PAD*2
    @h = @icon.h + PAD*2
    super
    @hi.put(@icon, PAD, PAD)
    @unhi.put(@icon, PAD, PAD)
    @parent.resized = true
  end
end

class RadioButtons < Hbox
  def initialize(hash={})
    super(hash)
    @radio_buttons = []
  end
  def add(*kids)
    super(*kids)
    for kid in kids
      if kid.kind_of?(StickyButton)
        @radio_buttons << kid
        def kid.on_selected(event)
          @parent.new_radio(self)
        end
      end
    end
  end
  def new_radio(child)
    for radio in @radio_buttons
      if radio.active? and radio != child
        radio.active = false
      end
    end
  end
  # returns selected widget or nil
  def selected
    @radio_buttons.find { |b| b.active? }
  end
end

class HToolBar < RadioButtons
end
