# TODO: respect max in both directions
module HLayout
  # returns new width and height after arranging horizontally
  def arrange(kids, fix_w)
    newx = 0
    newy = 0
    max_w = 0
    max_h = 0
    for child in kids
      # push to next row
      if fix_w > 0 and child.w + newx > fix_w
        newx = 0
        newy = max_h
      end
      child.x = newx
      child.y = newy
      newx += child.w
      max_w = newx if newx > max_w
      max_h = (child.h + newy) if (child.h + newy) > max_h
    end
    [max_w, max_h]
  end
end

module VLayout
  def arrange(kids, fix_h)
    newx = 0
    newy = 0
    max_w = 0
    max_h = 0
    for child in kids
      # push to next column
      if fix_h > 0 and child.h + newy > fix_h
        newy = 0
        newx = max_w
      end
      child.y = newy
      child.x = newx
      newy += child.h
      max_w = (child.w + newx) if (child.w+newx) > max_w
      max_h = newy if newy > max_h
    end
    [max_w, newy]
  end
end

class Layout < Widget
  def initialize(hash={})
    super(hash)
    @fixw, @fixh = @w, @h
    @oldw, @oldh = @w, @h
  end
  def draw(root_surf, dx, dy, dw, dh)
    return unless realized? and dirty?
    root_surf.fill_rect(dx+@x, dy+@y, dw, dh, 0)
    @children.each do |child|
      child.dirty
      child.draw(root_surf, dx+@x, dy+@y)
    end
    clean
  end
end

# these are fixed size!
class Hbox < Layout
  include HLayout
  def resize
    @oldw, @oldh = @w, @h
    @w, @h = arrange(@children, @fixw)
    self
  end
  def draw(s, dx, dy)
    super(s, dx, dy, @fixw, @oldh)
  end
end

class Vbox < Layout
  include VLayout
  def resize
    @oldw, @oldh = @w, @h
    @w, @h = arrange(@children, @fixh)
    self
  end
  def draw(s, dx, dy)
    super(s, dx, dy, @oldw, @fixh)
  end
end
