# extend in c?
class TextInput < Widget
  PAD = 4
  BORDER = 1
  BCOLOR = 0x0000FF00
  FILL = 0x00006600

  # the height is controlled by the font size
  def initialize(hash={})
    super(hash)
    @buff = ""
    @color = (hash[:color] or [255,255,255])
    @size = (hash[:size] or 14)
    @textwidth = (hash[:textwidth] or 10)
    connect_method(SDL::Event2::KeyDown, :on_key_down)
    connect_method(SDL::Event2::MouseButtonDown, :on_mouse_down)
    @upcase = false
  end

  def init_sdl
    @font = $game.load_font('font_bold', @size)
    chrw, @h = @font.text_size("M")
    @h += PAD*2+BORDER*2
    @w = chrw * @textwidth + PAD*2+BORDER*2
    @surface = create_surface(@w, @h)

    # fill up the text area
    @surface.fill_rect(BORDER,BORDER,@w-BORDER*2,@h-BORDER*2,FILL)
    draw_border(@surface, BCOLOR, BORDER)
    @parent.resized = true
  end

  # implementations should use this
  def on_enter
    unfocus
    clear
  end

  def clear
    @buff = ""
    key_unfocus
    redraw(cursor="")
  end
  def buff=(new)
    @buff = new
    redraw
  end
  def add_chr(chr)
    new = @buff + chr
    dw, dh = @font.text_size(new)
    return if dw > @w - 4*PAD+2*BORDER
    @buff = new
    redraw
  end
  def backspace
    return if @buff.size == 0
    @buff[-1] = ""
    redraw
  end

  def redraw(cursor="_")
    @surface.fill_rect(BORDER,BORDER,@w-BORDER*2,@h-BORDER*2,FILL)
    @font.drawSolidUTF8(@surface, @buff + cursor, PAD*2, PAD, *@color)
    dirty
  end

  # to select this widget for typing
  def select
    key_focus
    @font.drawSolidUTF8(@surface, @buff + "_", PAD*2, PAD, *@color)
    dirty
  end

  def on_mouse_down(event)
    select
  end

  # this is ugly.. SDL gives the unshifted key. we must translate this
  # to a shifted version if the shift key is down or capslock is active
  def on_key_down(event)
    if event.sym >= ?a and event.sym <= ?z
      if upcase?(event.mod) then add_chr(event.sym.chr.upcase)
      else add_chr(event.sym.chr)
      end
    elsif event.sym == SDL::Key::SPACE
      add_chr(" ")
    elsif event.sym == SDL::Key::BACKSPACE
      backspace
    elsif event.sym <= 57 and event.sym >= 48
      # numbers 0 through 9
      puts shift?(event.mod)
      if shift?(event.mod)
        case event.sym
        when ?1 then add_chr('!')
        when ?2 then add_chr('@')
        when ?3 then add_chr('#')
        when ?4 then add_chr('$')
        when ?5 then add_chr('%')
        when ?6 then add_chr('^')
        when ?7 then add_chr('&')
        when ?8 then add_chr('*')
        when ?9 then add_chr('(')
        when ?0 then add_chr(')')
        end
      else add_chr(event.sym.chr)
      end
    elsif event.sym == SDL::Key::RETURN
      # flush the buffer to any on_enter methods
      on_enter
    else # remainder: if not handled here, then ignore
      # TODO: some don't respond to caps lock!
      case event.sym
      when ?` then (shift?(event.mod) ? add_chr('~') : add_chr('`'))
      when ?- then (shift?(event.mod) ? add_chr('_') : add_chr('-'))
      when ?= then (shift?(event.mod) ? add_chr('+') : add_chr('='))
      when ?[ then (shift?(event.mod) ? add_chr('{') : add_chr('['))
      when ?] then (shift?(event.mod) ? add_chr('}') : add_chr(']'))
      when ?\\ then (shift?(event.mod) ? add_chr('|') : add_chr('\\'))
      when ?; then (shift?(event.mod) ? add_chr(':') : add_chr(';'))
      when ?' then (shift?(event.mod) ? add_chr('"') : add_chr('\''))
      when ?, then (shift?(event.mod) ? add_chr('<') : add_chr(','))
      when ?. then (shift?(event.mod) ? add_chr('>') : add_chr('.'))
      when ?/ then (shift?(event.mod) ? add_chr('?') : add_chr('/'))
      end
    end
  end

  def upcase?(mod)
    mod & (SDL::Key::MOD_CAPS|SDL::Key::MOD_LSHIFT|SDL::Key::MOD_RSHIFT) != 0
  end

  def shift?(mod)
    mod & (SDL::Key::MOD_LSHIFT|SDL::Key::MOD_RSHIFT) != 0
  end
end
