require 'ui/animations/pitdeath'
require 'ui/animations/laserfire'
require 'ui/animations/explode'

class Animate

  def initialize
    @thread = nil
    @mutex = Mutex.new
    @current_history = []
    # client side game history is here
    #@game_history = []
    @requests = []
    @log = Logger["bf::client::animation"]
    run_animation_thread
  end

  def init_sdl(root, widget)
    @root = root
    @widget = widget
    @surface = widget.surface
  end

  def run_animation_thread
    @thread = Thread.new do
      begin
        Thread.stop # careful!
        while true
          while @requests.size > 0
            req = nil
            @mutex.synchronize { req = @requests.shift }
            animate_turn(req)
            Thread.stop
          end
        end
      rescue Exception => e
        @log.fatal e
        raise e
      end
    end
    @thread.abort_on_exception = true
  end

  # animation controlls (from ui)
  def animate
    @mutex.synchronize { @requests << $game.turn_history }
    @thread.wakeup
  end

  # actual animation entry point (in thread)
  def animate_turn(turn)
    for snap in turn
      animate_segment(snap)
    end
  end
  def animate_segment(snapshot)
      @log.debug {snapshot}
      @old_delta = @delta
      @delta = snapshot.game_robots.collect{|r| $game.board.square_at(r.row, r.col)}
      @delta.uniq!
      animate_action(snapshot)
  end

  def animate_action(snapshot)
    @widget.robots = snapshot.game_robots
    case snapshot.action
    when Action::PITDEATH then animate_pitdeath(snapshot)
    when Action::REXPLODE, Action::CRUSH then animate_explode(snapshot)
    when Action::LASERFIRE 
      animate_laserfire(snapshot)
      # TODO: update ui damage, locked registers, etc
    else
      animate_move(snapshot)
    end
    time = Action::Delays[snapshot.action]
    sleep (time.nil? ? 0 : time)
  end

  def animate_move(snapshot)
    @old_delta.each {|s| s.dirty} if @old_delta
    @delta.each {|s| s.dirty }
    @widget.dirty
  end

end
