# actions.rb has the client-specific actions
# this file just has the gui definition

class ClientUI < MainWindow
  attr_reader :client, :registers, :boardui
  attr_accessor :sidebox
  @@top = nil
  DEVSIZE = 6

  def initialize(uri)
    @@top = self

    $game = ClientGameState.new

    @client = GameClient.instance
    @client.serveruri = uri
    @client.ui = self

    side = 50*3 + Button::PAD * 6
    under = 75 + 10
    bdsize =  DEVSIZE * $game.theme['square_size']

    args = {
      :w=> bdsize + side,
      :h=> bdsize + under + 33,
      :label=>"BotFrenzy"
    }
    super(args)

    create_title_bar

    args[:x] = 0
    args[:y] = 33
    args[:w] = bdsize 
    args[:h] = bdsize
    @boardui = BoardView.new(args)

    args[:w] = side
    args[:h] = 75*3 + Button::PAD*6
    args[:x] = bdsize
    args[:y] = 33
    args[:label] = "sidebox"
    @sidebox = Hbox.new(args)

    @statusbox = StatusBox.new(:x=>bdsize,:y=>33 + 75*3 + Button::PAD*6)

    @registers = RegisterWidget.new(:x=>0, :y=>bdsize+33)

    submit = LabelButton.new(:label=>"Submit", :x=>@registers.w, :y=>bdsize+33)
    def submit.on_selected(event)
      @@top.handle_play
    end

    add @boardui, @sidebox, @statusbox, @registers, submit
  end

  def create_title_bar
    bar = Hbox.new
    b1 = LabelButton.new(:label=>"Server")
    def b1.on_selected(event)
      return if @@top.client.connected?
      @@top.client.connect
    end
    b2 = LabelButton.new(:label=>"Create")
    def b2.on_selected(event)
      return unless @@top.client.connected?
      return if $game.gametype
      @@top.client.create_game
    end
    b3 = LabelButton.new(:label=>"Replay")
    def b3.on_selected(event)
      return unless $game.turn_history.size > 0
      @@top.boardui.animate.animate
    end
    b4 = LabelButton.new(:label=>"Quit")
    def b4.on_selected(event)
      @@top.quit
    end
    bar.add b1,b2,b3,b4
    add bar
  end

end
