# try to generalize the ui here
class ClientUI < MainWindow
  attr_reader :client, :registers, :boardui
  attr_accessor :sidebox, :standalone
  @@top = nil
  DEVSIZE = 6

  def initialize(uri=nil)
    @@top = self

    $game = ThemeState.new
    @standalone = nil

   side = 50*3 + Button::PAD * 6
    under = 75 + 10
    bdsize =  DEVSIZE * $game.theme['square_size']

    args = {
      :w=> bdsize + side,
      :h=> bdsize + under + 33,
      :label=>"BotFrenzy"
    }
    super(args)

    create_title_bar

    args[:x] = 0
    args[:y] = 33
    args[:w] = bdsize 
    args[:h] = bdsize
    @boardui = BoardView.new(args)

    args[:w] = side
    args[:h] = 75*3 + Button::PAD*6
    args[:x] = bdsize
    args[:y] = 33
    args[:label] = "sidebox"
    @sidebox = Hbox.new(args)

    @statusbox = StatusBox.new(:x=>bdsize,:y=>33 + 75*3 + Button::PAD*6)

    @registers = RegisterWidget.new(:x=>0, :y=>bdsize+33)

    @submit = LabelButton.new(:label=>"Submit", :x=>@registers.w, :y=>bdsize+33)
    def @submit.on_selected(event)
      if @@top.network_game? then @@top.handle_play
      else @@top.handle_standalone_play
      end
    end

    add @boardui, @sidebox, @statusbox, @registers, @submit
  end

  def create_title_bar
    bar = Hbox.new
    b1 = LabelButton.new(:label=>"Server")
    def b1.on_selected(event)
      @@top.connect_server
    end
    b2 = LabelButton.new(:label=>"Create")
    def b2.on_selected(event)
      @@top.create_game
    end
    b3 = LabelButton.new(:label=>"Replay")
    def b3.on_selected(event)
      return unless $game.turn_history.size > 0
      @@top.boardui.animate.animate
    end
    b4 = LabelButton.new(:label=>"Quit")
    def b4.on_selected(event)
      @@top.quit
    end
    bar.add b1,b2,b3,b4
    add bar
  end

  def network_game?
    not @standalone and @client and @client.connected?
  end

  def create_game
    if network_game?
      return if $game.gametype
      @client.create_game
    else # single game state 
      @@top.create_standalone_game
    end
  end


  def connect_server
    return if network_game?
    @client = GameClient.instance
    $game = ClientGameState.new
    @client.serveruri = uri
    @client.ui = self
    @client.connect
  end

  ############# standalone stuff
  
  # there should be a window to enter data
  def create_standalone_game
    # TODO: once the game is over, set @standalone to false!
    return if @standalone
    @standalone = true
    @client = StandaloneGame.new(self)
    $game = GameState.new
    players = %w{Zapf Clawbot Hammerbot}
    $game.gametype = LastRobotStanding.new
    $game.xml_board, $game.board =  BoardFactory.create_from_file("boards/game_board.xml")
    @boardui.game_created

    @client.create_game(players)
    @boardui.start_game
  end

  def handle_standalone_play
    return unless @registers.numcards == 5
    @client.recv_play_hand($game.robot.name, @registers.collect_play)
    @activerobot += 1
    return if @activerobot == $game.game_robots.size
    $game.robot = $game.game_robots[@activerobot]
    draw_hand($game.robot)
    @statusbox.update
  end

  # standalone: recieve all player hands
  # and draw the initial robots
  def recv_hands
    @activerobot = 0
    $game.robot = $game.game_robots[@activerobot]
    draw_hand($game.robot)
    @statusbox.update
  end
end
